unit RS232_Relaisboard;

interface

type
  TRS232_Relaisboard = class
    procedure Connect;
    procedure Disconnect;
    function InitBoards: boolean;
    function SetRelais(Board, Relais, Status: integer): boolean;
    procedure SetRelaisDummy(Board, Relais, Status: integer);
    function ResumeDummySettings(Board: integer): boolean;
    function Zerobyte(Board: integer): boolean;
    function NOPCheck: boolean;
  end;

implementation

uses RS232, Main, SysUtils;

const
  BoardOffset: integer = 0;
  SwitchTime: integer  = 10;

var
  BoardStatus: array[1..2] of integer;
  ComOpen:     boolean;

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Frame senden'}

function SendFrame(Byte1, Byte2, Byte3: integer): boolean;
var
  Checksum, ReturnByte1, ReturnByte2, ReturnByte3, ReturnByte4: integer;
begin

  checksum := byte1 xor byte2 xor byte3;

  SendByte(Byte1);
  SendByte(Byte2);
  SendByte(Byte3);
  SendByte(Checksum);

  ReturnByte1 := ReadByte;
  ReturnByte2 := ReadByte;
  ReturnByte3 := ReadByte;
  ReturnByte4 := ReadByte;

  Checksum := ReturnByte1 xor ReturnByte2 xor ReturnByte3;

  if (checksum = ReturnByte4) and (ReturnByte1 <> -1) and (ReturnByte2 <> -1) and
    (ReturnByte3 <> -1) then
    SendFrame := True
  else
  begin
    SendFrame := False;
  end;

end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Initialisierung'}

procedure TRS232_Relaisboard.Connect;
begin
  if ComOpen = False then
  begin
    OPENCOM('COM2: baud=19200 data=8 parity=N stop=1');
    ComOpen := True;
  end;
end;

procedure TRS232_Relaisboard.Disconnect;
begin
  if ComOpen then
  begin
    CLOSECOM;
    ComOpen := False;
  end;
end;

function TRS232_Relaisboard.InitBoards: boolean;
begin
  InitBoards := SendFrame(1, 1 + BoardOffset, 0);
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Board-Funktionen'}

function TRS232_Relaisboard.SetRelais(Board, Relais, Status: integer): boolean;
var
  Relais_address, Board_state, New_board_state: integer;
  Success: boolean;
begin

  Main.Form1.StatusMessage('Setze Relais ' + IntToStr(Relais) +
    ' auf Board' + IntToStr(Board) + ': ' + IntToStr(Status));

  Relais_address := 1;

  case Relais of
    1: Relais_address := 1;
    2: Relais_address := 2;
    3: Relais_address := 4;
    4: Relais_address := 8;
    5: Relais_address := 16;
    6: Relais_address := 32;
    7: Relais_address := 64;
    8: Relais_address := 128;
  end;

  Board_state := BoardStatus[Board];

  if Status = 0 then
    New_board_state := Board_state and 65535 - Relais_address
  else
    New_board_state := Board_state or Relais_address;

  Success := SendFrame(3, Board + BoardOffset, New_board_state);

  if success then
  begin
    BoardStatus[Board] := New_board_state;
    Main.Form1.SetRelaisGUIState(Board, Relais, Status);
  end;

  SetRelais := Success;

  Sleep(SwitchTime);

end;

procedure TRS232_Relaisboard.SetRelaisDummy(Board, Relais, Status: integer);
var
  Relais_address, Board_state, New_board_state: integer;
begin

  Main.Form1.StatusMessage('Setze Relais ' + IntToStr(Relais) +
    ' auf Board' + IntToStr(Board) + ': ' + IntToStr(Status) + ' (Dummy)');

  Relais_address := 1;

  case Relais of
    1: Relais_address := 1;
    2: Relais_address := 2;
    3: Relais_address := 4;
    4: Relais_address := 8;
    5: Relais_address := 16;
    6: Relais_address := 32;
    7: Relais_address := 64;
    8: Relais_address := 128;
  end;

  Board_state := BoardStatus[Board];

  if Status = 0 then
    New_board_state := Board_state and 65535 - Relais_address
  else
    New_board_state := Board_state or Relais_address;

  BoardStatus[Board] := New_board_state;
  Main.Form1.SetRelaisGUIState(Board, Relais, Status + 2);

end;

function TRS232_Relaisboard.ResumeDummySettings(Board: integer): boolean;
var
  Success: boolean;
  I, status, rest, division: integer;
begin

  Main.Form1.StatusMessage('bernehme Dummy-Status auf Board ' + IntToStr(Board));

  Success := SendFrame(3, Board + BoardOffset, BoardStatus[Board]);

  status := BoardStatus[Board];

  if Success then
  begin
    for I := 1 to 8 do
    begin
      rest     := status div 2;
      division := status mod 2;
      Main.Form1.SetRelaisGUIState(Board, I, division);
      status := rest;
    end;
  end;

  ResumeDummySettings := success;

  Sleep(SwitchTime);

end;

function TRS232_Relaisboard.Zerobyte(Board: integer): boolean;
var
  Success: boolean;
begin

  Main.Form1.StatusMessage('Alle Relais aus');

  Success := SendFrame(3, Board + BoardOffset, 0);

  if Success then
  begin
    BoardStatus[Board] := 0;
    Main.Form1.SetRelaisGUIStateZerobyte;
  end;

  Zerobyte := Success;

  Sleep(SwitchTime);

end;

function TRS232_Relaisboard.NOPCheck: boolean;
begin

  Main.Form1.StatusMessage('NOP-Test');

  NOPCheck := SendFrame(0, 1, 0);

end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

end.
