unit Machine;

interface

type
  TMachine = class
    procedure SetMCUTemp(Value: integer);
    procedure SetMCUIn(Value: integer);
    procedure SetMCUInSum(Value: integer);
    procedure SetMCUWWState(Value: integer);
    procedure SetMCUWWTime(Value: integer);

    function GetGUITemp: string;
    function GetGUIIn: string;
    function GetGUIVerbrauch: string;
    function GetGUIFuellstand: string;
    function GetGUIWWTime: string;

    function GetTemp: real;
    function GetIn: real;
    function GetInSum: real;
    function GetFuellstand: real;
    function GetMCUIn: integer;
    function GetMCUInSum: integer;
    function GetMCUTemp: integer;
    function GetWWState: boolean;
    function GetWWTime: integer;
    function GetInEn: boolean;

    procedure Machine_Kuehlung(Status: boolean);
    procedure Machine_Anlauf(Status: boolean);
    procedure Machine_Abpumpen(Status: boolean);
    procedure Machine_Reiniger(Status: boolean);
    procedure Machine_Heizen(Status: boolean);
    procedure Machine_Umwaelzen(Status: boolean);
    procedure Machine_Regenerieren(Status: boolean);
    procedure Machine_Zulauf(Status: boolean);
    procedure Machine_Korbwahl(Status: boolean);
    procedure Machine_Alarm(Status: boolean);
    procedure Machine_AllOff;

    function GetKuehlungState: boolean;
    function GetAbpumpenState: boolean;
    function GetReinigerState: boolean;
    function GetHeizenState: boolean;
    function GetUmwaelzenState: boolean;
    function GetRegenerierenState: boolean;
    function GetZulaufState: boolean;
    function GetKorbwahlState: boolean;
    function GetAlarmState: boolean;
    function GetAnlaufState: boolean;

    procedure ResetFuellstand;
    procedure ResetIn;
    procedure Enable_In;
    procedure Disable_In;
    procedure Reset_WWTime;
  end;

implementation

uses Main, SysUtils, Math;

const
  AnlaufInterval: integer = 2000;
  KuehlungNachlaufInterval: integer = 60000;

var
  MCUTemp, MCUIn, MCUInSum, MCUWWTime: integer;
  MCUWWState, MCUInEn: boolean;
  RealTemp, RealIn, RealInSum, RealFuellstand: real;

  MachineStatus_Abpumpen, MachineStatus_Reiniger, MachineStatus_Heizen,
  MachineStatus_Umwaelzen, MachineStatus_Regenerieren, MachineStatus_Zulauf,
  MachineStatus_Korbwahl, MachineStatus_Alarm, MachineStatus_Kuehlung,
  MachineStatus_Anlauf: boolean;

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Private Funktionen'}

function StatusKonvert(Status: boolean): string;
begin
  if Status = True then
    StatusKonvert := '1'
  else
    StatusKonvert := '0';
end;

procedure UmwaelzMotorAnlauf(Status: boolean);
begin
  Main.Form1.MachineAnlaufTimer.Interval := AnlaufInterval;
  Main.Form1.MachineAnlaufTimer.Enabled  := Status;
  Main.Form1.Machine.Machine_Anlauf(Status);
end;

procedure SteuerungsKuehlung(Status: boolean);
begin
  if Status then
  begin
    Main.Form1.Machine.Machine_Kuehlung(True);
  end
  else
  begin
    Main.Form1.MachineLuefterNachlaufTimer.Interval := KuehlungNachlaufInterval;
    Main.Form1.MachineLuefterNachlaufTimer.Enabled  := True;
  end;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'SetMCU'}

procedure TMachine.SetMCUTemp(Value: integer);
begin
  MCUTemp  := Value;
  //RealTemp:=-9.394641E-5*(MCUTemp*MCUTemp)+0.048556*MCUTemp+59.941339;
  RealTemp := ((20203.55220) / (Power(1.00314, MCUTemp) + 74.92836)) - 192.80725;
end;

procedure TMachine.SetMCUIn(Value: integer);
begin
  if ((Value - MCUIn) / 136.7)>=0 then  
    RealFuellstand := RealFuellstand + ((Value - MCUIn) / 136.7);
  MCUIn  := Value;
  RealIn := MCUIn / 136.7;
end;

procedure TMachine.SetMCUInSum(Value: integer);
begin
  MCUInSum  := Value;
  RealInSum := MCUInSum / 136.7;
end;

procedure TMachine.SetMCUWWState(Value: integer);
begin
  if Value = 1 then
    MCUWWState := True
  else
    MCUWWState := False;
end;

procedure TMachine.SetMCUWWTime(Value: integer);
begin
  MCUWWTime := Value * 10;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'GetGUI'}

function TMachine.GetGUITemp: string;
begin
  GetGUITemp := FloatToStrF(RealTemp, ffNumber, 15, 2) + ' C';
end;

function TMachine.GetGUIIn: string;
begin
  GetGUIIn := FloatToStrF(RealIn, ffNumber, 15, 2) + ' l';
end;

function TMachine.GetGUIVerbrauch: string;
begin
  GetGUIVerbrauch := FloatToStrF(RealInSum, ffNumber, 15, 2) + ' l';
end;

function TMachine.GetGUIFuellstand: string;
begin
  GetGUIFuellstand := FloatToStrF(RealFuellstand, ffNumber, 15, 2) + ' l';
end;

function TMachine.GetGUIWWTime: string;
begin
  GetGUIWWTime := IntToStr(MCUWWTime);
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Get}

function TMachine.GetTemp: real;
begin
  GetTemp := RealTemp;
end;

function TMachine.GetIn: real;
begin
  GetIn := RealIn;
end;

function TMachine.GetInSum: real;
begin
  GetInSum := RealInSum;
end;

function TMachine.GetFuellstand: real;
begin
  GetFuellstand := RealFuellstand;
end;

function TMachine.GetMCUIn: integer;
begin
  GetMCUIn := MCUIn;
end;

function TMachine.GetWWState: boolean;
begin
  GetWWState := MCUWWState;
end;

function TMachine.GetWWTime: integer;
begin
  GetWWTime := MCUWWTime;
end;

function TMachine.GetMCUInSum: integer;
begin
  GetMCUInSum := MCUInSum;
end;

function TMachine.GetMCUTemp: integer;
begin
  GetMCUTemp := MCUTemp;
end;

function TMachine.GetInEn: boolean;
begin
  GetInEn := MCUInEn;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'Machine'}

procedure TMachine.Machine_Kuehlung(Status: boolean);
begin
  if MachineStatus_Kuehlung <> Status then
  begin
    Main.Form1.RELServerSocketWrite('SET_RELAIS;1;8;' + StatusKonvert(Status));
    MachineStatus_Kuehlung := Status;
  end;
end;

procedure TMachine.Machine_Anlauf(Status: boolean);
begin
  if MachineStatus_Anlauf <> Status then
  begin
    Main.Form1.RELServerSocketWrite('SET_RELAIS;1;6;' + StatusKonvert(Status));
    MachineStatus_Anlauf := Status;
  end;
end;

procedure TMachine.Machine_Abpumpen(Status: boolean);
begin
  if MachineStatus_Abpumpen <> Status then
  begin
    SteuerungsKuehlung(Status);
    Main.Form1.RELServerSocketWrite('SET_RELAIS;1;3;' + StatusKonvert(Status));
    MachineStatus_Abpumpen := Status;
  end;
end;

procedure TMachine.Machine_Reiniger(Status: boolean);
begin
  if MachineStatus_Reiniger <> Status then
  begin
    SteuerungsKuehlung(Status);
    Main.Form1.RELServerSocketWrite('SET_RELAIS;1;2;' + StatusKonvert(Status));
    MachineStatus_Reiniger := Status;
  end;
end;

procedure TMachine.Machine_Heizen(Status: boolean);
begin
  if MachineStatus_Heizen <> Status then
  begin
    SteuerungsKuehlung(Status);
    Main.Form1.RELServerSocketWrite('SET_RELAIS;1;1;' + StatusKonvert(Status));
    MachineStatus_Heizen := Status;
  end;
end;

procedure TMachine.Machine_Umwaelzen(Status: boolean);
begin
  if MachineStatus_Umwaelzen <> Status then
  begin
    UmwaelzMotorAnlauf(Status);
    Main.Form1.RELServerSocketWrite('SET_RELAIS_DUMMY;2;1;' + StatusKonvert(Status));
    Main.Form1.RELServerSocketWrite('SET_RELAIS_DUMMY;2;2;' + StatusKonvert(Status));
    Main.Form1.RELServerSocketWrite('RESUME_DUMMY;2');
    MachineStatus_Umwaelzen := Status;
  end;
end;

procedure TMachine.Machine_Regenerieren(Status: boolean);
begin
  if MachineStatus_Regenerieren <> Status then
  begin
    Main.Form1.RELServerSocketWrite('SET_RELAIS_DUMMY;2;3;' + StatusKonvert(Status));
    Main.Form1.RELServerSocketWrite('SET_RELAIS_DUMMY;2;4;' + StatusKonvert(Status));
    Main.Form1.RELServerSocketWrite('RESUME_DUMMY;2');
    MachineStatus_Regenerieren := Status;
  end;
end;

procedure TMachine.Machine_Zulauf(Status: boolean);
begin
  if MachineStatus_Zulauf <> Status then
  begin
    Main.Form1.RELServerSocketWrite('SET_RELAIS_DUMMY;2;5;' + StatusKonvert(Status));
    Main.Form1.RELServerSocketWrite('SET_RELAIS_DUMMY;2;6;' + StatusKonvert(Status));
    Main.Form1.RELServerSocketWrite('RESUME_DUMMY;2');
    MachineStatus_Zulauf := Status;
  end;
end;

procedure TMachine.Machine_Korbwahl(Status: boolean);
begin
  if MachineStatus_Korbwahl <> Status then
  begin
    Main.Form1.RELServerSocketWrite('SET_RELAIS_DUMMY;2;7;' + StatusKonvert(Status));
    Main.Form1.RELServerSocketWrite('SET_RELAIS_DUMMY;2;8;' + StatusKonvert(Status));
    Main.Form1.RELServerSocketWrite('RESUME_DUMMY;2');
    MachineStatus_Korbwahl := Status;
  end;
end;

procedure TMachine.Machine_Alarm(Status: boolean);
begin
  if MachineStatus_Alarm <> Status then
  begin
    Main.Form1.RELServerSocketWrite('SET_RELAIS;1;7;' + StatusKonvert(Status));
    MachineStatus_Alarm := Status;
  end;
end;

procedure TMachine.Machine_AllOff;
begin
  Main.Form1.RELServerSocketWrite('ZEROBYTE;1');
  Main.Form1.RELServerSocketWrite('ZEROBYTE;2');
  MachineStatus_Abpumpen  := False;
  MachineStatus_Reiniger  := False;
  MachineStatus_Heizen    := False;
  MachineStatus_Umwaelzen := False;
  MachineStatus_Regenerieren := False;
  MachineStatus_Zulauf    := False;
  MachineStatus_Korbwahl  := False;
  MachineStatus_Alarm     := False;
  MachineStatus_Kuehlung  := False;
  MachineStatus_Anlauf    := False;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'GetState'}

function TMachine.GetKuehlungState: boolean;
begin
  GetKuehlungState := MachineStatus_Kuehlung;
end;

function TMachine.GetAbpumpenState: boolean;
begin
  GetAbpumpenState := MachineStatus_Abpumpen;
end;

function TMachine.GetReinigerState: boolean;
begin
  GetReinigerState := MachineStatus_Reiniger;
end;

function TMachine.GetHeizenState: boolean;
begin
  GetHeizenState := MachineStatus_Heizen;
end;

function TMachine.GetUmwaelzenState: boolean;
begin
  GetUmwaelzenState := MachineStatus_Umwaelzen;
end;

function TMachine.GetRegenerierenState: boolean;
begin
  GetRegenerierenState := MachineStatus_Regenerieren;
end;

function TMachine.GetZulaufState: boolean;
begin
  GetZulaufState := MachineStatus_Zulauf;
end;

function TMachine.GetKorbwahlState: boolean;
begin
  GetKorbwahlState := MachineStatus_Korbwahl;
end;

function TMachine.GetAlarmState: boolean;
begin
  GetAlarmState := MachineStatus_Alarm;
end;

function TMachine.GetAnlaufState: boolean;
begin
  GetAnlaufState := MachineStatus_Anlauf;
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

{$REGION 'MCU-Set'}

procedure TMachine.ResetFuellstand;
begin
  //SetMCUIn(0);
  RealFuellstand:=0;
end;

procedure TMachine.ResetIn;
begin
  MCUIn:=0;
  RealIn:=0;
  Main.Form1.MCUServerSocketWrite('RESET_IN');
end;

procedure TMachine.Enable_In;
begin
  Main.Form1.MCUServerSocketWrite('ENABLE_IN');
  MCUInEn := True;
end;

procedure TMachine.Disable_In;
begin
  Main.Form1.MCUServerSocketWrite('DISABLE_IN');
  MCUInEn := False;
end;

procedure TMachine.Reset_WWTime;
begin
  Main.Form1.MCUServerSocketWrite('RESET_WW_TIME');
end;

{$ENDREGION}

////////////////////////////////////////////////////////////////////////////////

initialization
  MCUInEn := True;

end.
